<?php

namespace Form;

use Form\Constraint\Constraint;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class Field
 * @package Form
 */
class Field
{

    /**
     * Input name
     * @var string
     */
    protected $name;

    /**
     * @var array
     */
    protected $messages;

    /**
     * @var Constraint[]
     */
    protected $constraints;

    /**
     * Field constructor.
     * @param $name
     * @param array $constraints
     * @param array $messages
     */
    public function __construct($name, $constraints = array(), $messages = array())
    {
        $this->name = $name;
        $this->constraints = $constraints;
        $this->messages = $messages;
    }

    /**
     * @param $name
     * @param Constraint $constraint
     * @param string $message
     * @return $this
     */
    public function addConstraints($name, Constraint $constraint, $message = '')
    {
        $this->constraints[$name] = $constraint;
        $this->messages[$name] = $message;

        return $this;
    }

    /**
     * @param $name
     * @param $message
     * @return $this
     */
    public function addMessage($name, $message)
    {
        $this->messages[$name] = $message;
        return $this;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param Request $request
     * @return array
     */
    public function isValid(Request $request)
    {
        $errors = array();
        foreach ($this->constraints as $name => $constraint) {
            if (in_array('Form\\RequestAwareInterface', class_implements($constraint))) {
                $constraint->setRequest($request);
            }

            $result = $constraint->isValid($request->get($this->getName()));

            if ($result == false) {
                $errors[] = (!empty($this->messages[$name])) ? $this->messages[$name] : $name;
            }
        }

        return $errors;
    }
}
